#include <stdio.h>
#include <stdlib.h>
#include <tcl.h>
#include <tk.h>
#include <string.h>

#include "fly.h"

/*
 * Declarations for various library procedures and variables (don't want
 * to include tkInt.h or tkConfig.h here, because people might copy this
 * file out of the Tk source directory to make their own modified versions).
 * - Taken from mkMain.c
 */



/*
 * Global variables used by the main program:
 */

char           *tcl_RcFileName = NULL;  /* Name of a user-specific startup
                                         * script to source if the
                                         * application is being run
                                         * interactively (e.g. "~/.wishrc").
                                         * Set by Tcl_AppInit.  NULL means
                                         * don't source anything ever. */
static int      tty;            /* Non-zero means standard input is a
                                 * terminal-like device.  Zero means it's a
                                 * file. */
static char     errorExitCmd[] = "exit 1";

/*
 * Command-line options:
 */

#if 0
static int      synchronize = 0;

static char    *fileName = NULL;
static char    *name = NULL;
static char    *display = NULL;
static char    *geometry = NULL;
#endif

/*
 * Forward declarations for procedures defined later in this file:
 */

#if 0
static void Prompt _ANSI_ARGS_((Tcl_Interp * interp, int partial));
static void StdinProc _ANSI_ARGS_((ClientData clientData,
                      int mask));
extern int      videoframecmd(ClientData clientData, Tcl_Interp * interp, int argc,
                   char *argv[]);

#endif

int             main(int argc, char **argv)
{
   extern int      newimage;
   extern int      new_view_parameter;
   extern Viewer  *viewer;
   Tcl_Interp     *interp;      /* Interpreter for this application. */
   ParentWindow   *parentwindow;
   char           *msg;

   if ((parentwindow = (ParentWindow *) malloc(sizeof(ParentWindow))) == NULL)
   {
      fprintf(stderr, "Out of memory!\n");
      exit(1);
   }

   parentwindow->interp = interp = Tcl_CreateInterp();
   parentwindow->name = strdup("NPAC fly-by demo");

   /*
    * If a display was specified, put it into the DISPLAY environment
    * variable so that it will be available for any sub-processes created by
    * us.
    */
   parentwindow->displayname = NULL;
/*
    if (display != NULL) {
        Tcl_SetVar2(interp, "env", "DISPLAY", displayname, TCL_GLOBAL_ONLY);
    }
*/

   /*
    * Initialize the Tk application.
    */

   if ((parentwindow->tkwin = Tk_CreateMainWindow(interp,
                                                  parentwindow->displayname,
                                                  parentwindow->name, "Tk"))
       == NULL)
   {
      fprintf(stderr, "%s\n", interp->result);
      exit(1);
   }

#if 0
#if 0
   _Xdebug = 1;
#endif
   XSynchronize(Tk_Display(parentwindow->tkwin), True);
#endif

   parentwindow->width = 694;
   parentwindow->height = 749;
   Tk_GeometryRequest(parentwindow->tkwin, parentwindow->width,
                      parentwindow->height);

   parentwindow->display = Tk_Display(parentwindow->tkwin);
   parentwindow->visual = Tk_Visual(parentwindow->tkwin);

   /*
    * Make command-line arguments available in the Tcl variables "argc" and
    * "argv".  Also set the "geometry" variable from the geometry specified
    * on the command line.
    */
/*
    if (geometry != NULL) {
        Tcl_SetVar(interp, "geometry", geometry, TCL_GLOBAL_ONLY);
    }
*/

   /*
    * Set the "tcl_interactive" variable.
    */

   Tcl_SetVar(interp, "tcl_interactive", isatty(0) ? "1" : "0",
              TCL_GLOBAL_ONLY);

   /*
    * Initialize tcl and tk
    */
   if (Tcl_Init(interp) == TCL_ERROR)
   {
      fprintf(stderr, "Tcl_Init failed: %s\n", interp->result);
   }
   if (Tk_Init(interp) == TCL_ERROR)
   {
      fprintf(stderr, "Tk_Init failed: %s\n", interp->result);
   }

   if (Photo_Init(interp) == TCL_ERROR)
   {
      fprintf(stderr, "Photo_Init failed: %s\n", interp->result);
   }

   Tcl_CreateCommand(interp, "viewer", (Tcl_CmdProc *) viewercmd,
                     (ClientData *) parentwindow, (Tcl_CmdDeleteProc *) NULL);
   /*
    * Invoke the script specified
    */

   printf("source fly.tcl\n");
   if (Tcl_VarEval(interp, "source fly.tcl", (char *) NULL) == TCL_OK)
   {
      tty = 0;
      fflush(stdout);

      /*
       * Loop infinitely, waiting for commands to execute.  When there are no
       * windows left, Tk_MainLoop returns and we exit.
       */

      while (tk_NumMainWindows > 0) {
         Tk_DoOneEvent(TK_DONT_WAIT);

         if (newimage == TRUE)
         {
            Tk_DoWhenIdle(UpdateViewer, (ClientData) viewer);
            newimage = FALSE;
         }

         if (new_view_parameter == TRUE)
         {
            Tk_DoWhenIdle(compute_view_info, (ClientData) viewer);
            new_view_parameter = FALSE;
         }
      }
      /*
       * Don't exit directly, but rather invoke the Tcl "exit" command. This
       * gives the application the opportunity to redefine "exit" to do
       * additional cleanup.
       */

      Tcl_Eval(interp, "exit");
      exit(0);
   }
   else
   {
      if ((msg = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY)) == NULL)
         msg = interp->result;
      fprintf(stderr, "%s\n", msg);
      Tcl_Eval(interp, errorExitCmd);
      exit(1);          /* Needed only to prevent compiler warnings. */
   }
}
